formula_mapper = {
    "Cyclopropane": "C3H6",
    "Pentane": "C5H12",
    "Kapton": "C22H10N2O5",
    "Mylar": "C10H8O4",
    "Polypropylene": "C3H6",
    "Butane": "C4H10",
    "Polystyrene": "C8H8",
    "Acetylene": "C2H2",
    "Polyvinyltoluene": "C9H10",
    "C amorphous": "C",
    "Methane": "NH3",
    "Hydroxy apatite": "Ca5P3O13H",
    "H2O solid": "H2O",
    "H2o liq": "H2O",
    "H2O vapor": "H2O",
    "acetylene": "C2H2",
    "Ethylene": "C2H4",
    "Cyclopropane": "C3H6",
    "Pentane": "C5H12",
    "Kapton": "C22H10N2O5",
    "Mylar": "C10H8O4",
    "Polypropylene": "C3H6",
    "Propylene": "C3H6",
    "Butane": "C4H10",
    "Polysterene": "C8H8",
    "Acetylene": "C2H2",
    "Polyvinyltoluene": "C9H10",
    "C amorphous": "C",
    "Methane": "NH3",
    "Hydroxy apatite": "Ca5P3O13H",
    "H2O solid": "H2O",
    "H2o liq": "H2O",
    "H2O vapor": "H2O",
    "acetylene": "C2H2",
    "heptane": "C7H16",
    "Pentane": "C5H12",
    "ethane": "C2H6",
    "Polysulfone": "C6H4SO2",
    "Methyl alcohol liquid, vapor": "CH4O",
    "Propyl alcohol liquid, vapor": "C3H8O",
    "Propyl alcohol vapor": "C3H8O",
    "Propylene oxide vapor": "C3H6O",
    "Cyclohexane liquid, vapor": "C6H12",
    "Graphite Oxide": "C5O5",
    "Ethyl alcohol liquid, vapor": "C2H6O",
    "Ethyl alcohol liquid": "C2H6O",
    "Ethane": "C2H6",
    "Polycarbonate": "C15H16O2",
    "Pentane liquid, vapor": "C5H12",
    "Hexane liquid, vapor": "C6H14",
    "Heptane liquid, vapor": "C7H16",
    "Propane": "C3H8",
    "Ethylene": "C2H4",
    "Polyethylene": "C2H4",
    "Havar": "Co",
    "c3f8": "C3F8",
    "Cyclopropane": "C3H6",
    "Pentane": "C5H12",
    "Kapton": "C22H10N2O5",
    "Mylar": "C10H8O4",
    "Polypropylene": "C3H6",
    "Butane": "C4H10",
    "Polystyrene": "C8H8",
    "Acetylene": "C2H2",
    "Polyvinyltoluene": "C9H10",
    "C amorphous": "C",
    "Methane": "NH3",
    "Hydroxy apatite": "Ca5P3O13H",
    "H2O solid": "H2O",
    "H2o liq": "H2O",
    "H2O vapor": "H2O",
    "acetylene": "C2H2",
    "Ethylene": "C2H4",
    "Cyclopropane": "C3H6",
    "Pentane": "C5H12",
    "Kapton": "C22H10N2O5",
    "Mylar": "C10H8O4",
    "Polypropylene": "C3H6",
    "Propylene": "C3H6",
    "Butane": "C4H10",
    "Polysterene": "C8H8",
    "Acetylene": "C2H2",
    "Polyvinyltoluene": "C9H10",
    "C amorphous": "C",
    "Methane": "NH3",
    "Hydroxy apatite": "Ca5P3O13H",
    "H2O solid": "H2O",
    "H2o liq": "H2O",
    "H2O vapor": "H2O",
    "acetylene": "C2H2",
    "heptane": "C7H16",
    "Pentane": "C5H12",
    "ethane": "C2H6",
    "Polysulfone": "C6H4SO2",
    "Methyl alcohol liquid, vapor": "CH4O",
    "Propyl alcohol liquid, vapor": "C3H8O",
    "Propyl alcohol vapor": "C3H8O",
    "Propylene oxide vapor": "C3H6O",
    "Cyclohexane liquid, vapor": "C6H12",
    "Graphite Oxide": "C5O5",
    "Ethyl alcohol liquid, vapor": "C2H6O",
    "Ethyl alcohol liquid": "C2H6O",
    "Ethane": "C2H6",
    "Polycarbonate": "C15H16O2",
    "Pentane liquid, vapor": "C5H12",
    "Hexane liquid, vapor": "C6H14",
    "Heptane liquid, vapor": "C7H16",
    "Propane": "C3H8",
    "Ethylene": "C2H4",
    "Polyethylene": "C2H4",
    "c6h6v": "C6H6",
    "vmet": "CH5N",
    "Methylamine": "CH5N",
    "etam": "C2H7N",
    "Ethylamine": "C2H7N",
    "ch3br": "CH3Br",
    "tmam": "C3H9N",
    "Trimethylamine": "C3H9N",
    "mam": "Methylamine",
    "Silicon nitride": "Si3N4",
    "br": "Br",
    "c3h8": "C3H8",
    "xev": "Xe",
    "baf2": "BaF2",
    "c4f8": "C4F8",
    "so2": "SO2",
    "sr": "Sr",
    "ch4": "CH4",
    "detev": "C4H10O",
    "Diethyl ether": "C4H10O",
    "Dimethyl ether": "C2H6O",
    "dmam": "C2H7N",
    "Dimethyl amine": "C2H7N",
    "cs": "Cs",
    "cd": "Cd",
    "cyhev": "C6H12",
    "Cyclohexane": "C6H12",
    "cycpr": "C3H6",
    "Cyclopropane": "C3H6",
    "rh": "Rh",
    "prpsv": "C3H6S",
    "Propylene sulfide": "C3H6S",
    "cs2v": "CS2",
    # "cs2v": "Carbon disulfide",
    "ch5br": "CH5Br",
    "ethsv": "C4H10S",
    "Ethyl sulfide": "C4H10S",
    "ccl2f": "CCl2F",
    "nd": "Nd",
    "cf3h": "CF3H",
    "pntov": "C5H10O",
    "3-Pentanone": "C5H10O",
    "3-Pentanone vapor": " C5H10O",
    "ccl4v": "CCl4",
    "xev": "Xe",
    "ethcl": "C2H5Cl",
    "EthylChloride": "C2H5Cl",
    "bacl2": "BaCl2",
    "zro2": "ZrO2",
    "l115": "LR-115",
    "cr39": "CR-39",
    "Ar solid": "Ar",
    "Hexane": "C6H14",
    "Octane": "C8H18",
    "Allene": "C3H4",
    "tefl": "C2F4",
    "c3f8": "C3F8",
    "Silicon Nitride": "Si3N4",
    "ars": "Ars",
    "inp": "InP",
    "CR-39": "C12H18O7",
    "caf2": "CaF2",
    "al2O3": "Al2O3",
    "n2o": "N2O",
    "Havar": "Co",
    "Diamond": "C",
    "Graphite": "C",
    "Al2o3": "Al2O3",
    "znte": "ZnTe",
    "inp": "InP",
    "c2brf": "C2BrF",
    "c2h3f": "C2H3Bf",
    "c2h3l": "C2H3",
    "c3f8": "C3F8",
    "c3h8": "C3H8",
    "c4f8": "C4F8",
    "c6h6v": "C6H6",
    "cbrf3": "CBrF3",
    "cclf3": "CClF3",
    "cdte": "CdTe",
    "cf2l2": "CF2Cl2",
    "cf3br": "CF3Br",
    "ch4": "CH4",
    "chclf": "CHClF",
    "Propylene": "C3H6",
    "Teflon": "C2F4",
    "Pentene": "C5H10",
    "Acetaldehyde": "C2H4O",
    "Acetone": "C3H6O",
    "C diamond": "C",
    "CCl4 liquid, vapor": "CCl4",
    "Dimethyl sulfite vapor": "C2H6O3S",
    "Dimethylamine": "C2H6NH",
    "Dimethyldisulfide vapor": "C2H6S2",
    "Dimethylsulfide vapor": "C2H6S",
    # 'Ethynylbenzene vapor',
    # 'Inp': 'InP',
    # 'LR-115',
    # 'NE-111',
    # 'P-dioxane': 'C4H8O2',
    # 'Pliolite',
    # 'Toluene',
    # 'c2h3f': 'C2H3F',
    # 'ce': 'Ce',
    # 'cl':'Cl',
    # 'clfm',
    # 'clfmv', 'co2c', 'cychv', 'cycov', 'cyhdv', 'cyhov', 'dclm',
    # 'dclmv', 'dfley', 'ealc', 'ealcv', 'etho', 'ethyl', 'form'
    # 'Vyns', 'air', 'alle',
    # 'btadv', 'but', 'butov', 'buty', 'c2h3f', 'ce', 'cl', 'clfm',
    # 'clfmv', 'co2c', 'cychv', 'cycov', 'cyhdv', 'cyhov', 'dclm',
    # 'dclmv', 'dfley', 'ealc', 'ealcv', 'etho', 'ethyl', 'form',
    # 'hptyv', 'mumet', 'phb', 'polys', 'pr', 'propy', 're', 'thphv',
    # 'tmsv
}



import torch.nn.functional as F
from torch import nn


class Model(nn.Module):

    def __init__(self,
                 num_features,
                 num_targets,
                 hidden_size1=10,
                 hidden_size2=24,
                 drop_rate1=0.2,
                 drop_rate2=0.5,
                 drop_rate3=0.5
                 ):
        super(Model, self).__init__()
        self.dense1 = nn.utils.weight_norm(nn.Linear(num_features, hidden_size1, bias=False))

        self.dropout2 = nn.Dropout(drop_rate2)
        self.dense2 = nn.utils.weight_norm(nn.Linear(hidden_size1, hidden_size2))

        self.dropout3 = nn.Dropout(drop_rate3)
        self.dense3 = nn.utils.weight_norm(nn.Linear(hidden_size2, 32))

        self.dropout4 = nn.Dropout(drop_rate3)
        self.dense4 = nn.utils.weight_norm(nn.Linear(32, hidden_size2))

        self.dropout5 = nn.Dropout(drop_rate3)
        self.dense5 = nn.utils.weight_norm(nn.Linear(hidden_size2, hidden_size1))

        self.batch_norm6 = nn.BatchNorm1d(hidden_size1)

        self.dense6 = nn.Linear(hidden_size1, num_targets)

    def forward(self, x):
        x = F.leaky_relu(self.dense1(x))

        x = self.dropout2(x)
        x = F.leaky_relu(self.dense2(x))

        x = self.dropout3(x)
        x = F.leaky_relu(self.dense3(x))

        x = self.dropout4(x)
        x = F.leaky_relu(self.dense4(x))

        x = self.dropout5(x)
        x = F.leaky_relu(self.dense5(x))

        x = self.dense6(x)

        return x



import numpy as np

from .inference import run_inference


def run_k_fold(folds, NFOLDS, seed, device, verbose=False, **kwargs):
    """Wraps run_inference to peform ensemble predictions"""

    valid_df = folds

    oof = np.zeros((len(valid_df), NFOLDS))

    for fold in range(NFOLDS):

        X_valid = valid_df[
            [
                "target_mass",
                "projectile_mass",
                "projectile_Z",
                "Z_max",
                "normalized_energy",
                "target_ionisation",
            ]
        ].values

        oof_ = run_inference(X_valid, fold, seed, device, verbose, **kwargs)

        oof[:, fold] = oof_[:, 0]

    return oof



import torch
from torch.utils.data.dataset import Dataset


class IaeaDataset(Dataset):
    def __init__(self, features, targets):

        self.features = features
        self.targets = targets

    def __len__(self):
        return self.features.shape[0]

    def __getitem__(self, idx):
        dct = {
            "x": torch.tensor(self.features[idx, :], dtype=torch.float),
            "y": torch.tensor(self.targets[idx, :], dtype=torch.float),
        }
        return dct


class TestDataset(Dataset):
    def __init__(self, features):
        self.features = features

    def __len__(self):
        return self.features.shape[0]

    def __getitem__(self, idx):
        dct = {"x": torch.tensor(self.features[idx, :], dtype=torch.float)}
        return dct


class InferenceDataset(Dataset):
    """Receives a dataframe containing the following features in this order
    - target atomic mass(amu)
    - projectile atomic mass (amu)
    - projectile Z
    - target Z
    - normalized stopping power (in MeV)
    """

    def __init__(self, features):

        self.features = features

    def __len__(self):
        return self.features.shape[0]

    def __getitem__(self, idx):
        dct = {
            "x": torch.tensor(self.features[idx, :], dtype=torch.float),
        }
        return dct



import os
import sys
import numpy as np
import pandas as pd
from pyvalem.formula import Formula
from ESPNN.data.formula_mapper import formula_mapper
import random
import torch


dir_path = os.path.dirname(os.path.realpath(__file__))


def generate_custom_table(
    projectile_name,
    target,
    minE,
    maxE,
    num_points,
):
    """
    Conveniency function to create an input table for the model.
    """

    min_powerE = np.log10(minE)
    max_powerE = np.log10(maxE)
    ener_range = np.logspace(
        min_powerE,
        max_powerE,
        num=num_points,
        endpoint=True,
        base=10.0,
        dtype=None,
        axis=0,
    )

    df = pd.DataFrame(
        {
            "projectile": num_points * [projectile_name],
            "target": num_points * [target],
            "normalized_energy": ener_range,
        }
    )

    return df


chem_prop = pd.read_csv(f"{dir_path}/data/input/chemicalProperties.csv")

chem_prop[" Symbol"] = chem_prop[" Symbol"].str.lstrip()

chem_prop_tab = chem_prop[[" Symbol", " Atomic_Number"]]

chem_prop_tab.set_index(" Symbol", inplace=True)

atomic_number_dict = chem_prop_tab.to_dict()[" Atomic_Number"]


def get_Z_projectile(name):

    try:

        return atomic_number_dict[str(name)]

    except:

        return np.nan


def get_mass(name):

    if name in formula_mapper.keys():
        name = formula_mapper[name]

    if name.lower() == "d2o":
        return 20

    if name.lower() == "d2o":
        return 4

    try:

        f = Formula(name)

        return f.mass

    except:

        return np.nan


def get_max_Z(name):

    if name in formula_mapper.keys():

        name = formula_mapper[name]

    try:

        f = Formula(name)

        return max([atomic_number_dict[str(atom)] for atom in f.atoms])

    except:

        return np.nan


def get_mass_atoms_ratio(name):

    if name in formula_mapper.keys():

        name = formula_mapper[name]
    if name.lower() == "d2o":
        return 20 / 3

    if name.lower() == "d2o":
        return 2

    try:

        f = Formula(name)

        return f.mass / f.natoms

    except:

        return np.nan


ion_prop = pd.read_table(f"{dir_path}/data/input/ionization_energies_wiki.txt")
ion_prop["Symbol"] = ion_prop["Symbol"].str.lstrip()

ion_prop_tab = ion_prop[["Symbol", "1st"]]

ion_prop_tab.set_index("Symbol", inplace=True)

ionisation_dict = ion_prop_tab.to_dict()["1st"]


def get_ionisation_projectile(name):

    name = str(name)

    target_dict = {
        "C amorphous": "C",
        "Graphite": "C",
        "O2": "O",
        "N2": "N",
        "H2": "H",
        "Havar": "Co",
    }

    if name in ["C amorphous", "O2", "N2", "H2", "Graphite", "Havar"]:

        name = target_dict[name]

    try:

        return ionisation_dict[str(name)]

    except:

        return np.nan


def seed_everything(seed=42):

    """
    Sets all the necessary seeds

    """

    random.seed(seed)
    # os.environ['PYTHONHASHSEED'] = str(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    torch.backends.cudnn.deterministic = True

    return None


elem_prop = pd.read_csv(f"{dir_path}/data/input/chemicalProperties.csv")

elem_prop[" Symbol"] = chem_prop[" Symbol"].str.lstrip()

elem_prop_tab = chem_prop[[" Symbol", " Atomic_Number"]]

elem_prop_tab.set_index(" Atomic_Number", inplace=True)

number_symbol_dict = elem_prop_tab[" Symbol"].to_dict()


def match_symbol_to_Z(num):

    try:

        return number_symbol_dict[int(num)]

    except:

        return "Unknown"



import sys
import argparse
from ESPNN.core import run_NN


class DefaultHelpParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n\n' % message)
        self.print_help()
        sys.exit(2)


if __name__ == "__main__":

    basic_usage = "python -m ESPNN [X] [Y]"
    custom_ener = " [-emin EMIN] [-emax EMAX] [-npoints NPOINTS] "
    custom_output = "[--plot PLOT] [--outdir OUTDIR]"
    usage = ''.join([basic_usage, custom_ener, custom_output])
    # parser = argparse.ArgumentParser(usage=usage, add_help=False)
    parser = DefaultHelpParser(usage=usage, add_help=False)

    parser.add_argument("X", help="Projectile name", type=str)
    parser.add_argument("Y", help="Target name", type=str)
    parser.add_argument(
        "-emin",
        dest="emin",
        default=0.001,
        type=float,
        help="Minimum energy value (default: 0.001)"
    )
    parser.add_argument(
        "-emax",
        dest="emax",
        default=10.,
        type=float,
        help="Maximum energy value (default: 10)"
    )
    parser.add_argument(
        "-npoints",
        dest="npoints",
        default=150,
        type=int,
        help="Number of grid points (default: 150)"
    )
    parser.add_argument(
        "--plot",
        dest="plot",
        default=True,
        help="Plot prediction (default: True)"
    )
    parser.add_argument(
        "--outdir",
        dest="outdir",
        default='./',
        help="Path to output folder (default: './')"
    )
    parser.add_argument(
        "-h",
        "--help",
        action="help",
        help="Show this message and exit"
    )

    args = parser.parse_args()

    run_NN(
        projectile=args.X,
        target=args.Y,
        emin=args.emin,
        emax=args.emax,
        npoints=args.npoints,
        outdir=args.outdir,
        plot=args.plot,
    )



import os
import pathlib

from setuptools import setup

# =============================================================================
# CONSTANTS
# =============================================================================

PATH = pathlib.Path(os.path.abspath(os.path.dirname(__file__)))

REQUIREMENTS = [
    "matplotlib==3.5.2",
    "numpy>=1.21.6",
    "pandas>=1.3.5",
    "pyvalem==2.5.7",
    "torch==1.11.0",
    "scikit-learn==0.24.2",
    "joblib>=1.0.0",
]

with open(PATH / "ESPNN" / "__init__.py") as fp:
    for line in fp.readlines():
        if line.startswith("__version__ = "):
            VERSION = line.split("=", 1)[-1].replace('"', "").strip()
            break

with open("README.md", "r") as readme:
    LONG_DESCRIPTION = readme.read()


# =============================================================================
# FUNCTIONS
# =============================================================================

setup(
    name="ESPNN",
    version=VERSION,
    description="Electronic Stopping Power Neural Network predictor",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author="""
    Felipe Bivort Haiek,
    Alejandra Mendez,
    Claudia Montanari,
    Darío Mitnik
    """,
    author_email="felipebihaiek@gmail.com, alemdz.7@gmail.com",
    url="https://github.com/ale-mendez/SPNN",
    packages=["ESPNN", "ESPNN.data"],
    include_package_data=True,
    install_requires=REQUIREMENTS,
    license="The GPLv3 License",
    keywords=[
        "Stopping Power",
        "Energy Loss",
        "Atoms",
        "Molecules",
        "Ions",
        "Machine Learning",
        "Neural Network",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering",
    ],
)



import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

from .utils import generate_custom_table
from .utils import get_ionisation_projectile
from .utils import get_mass
from .utils import get_max_Z
from .utils import get_Z_projectile
from .run_Kinf import run_k_fold

# Default HyperParameters:
NFOLDS = 5
SEED = np.random.randint(0, 12347, NFOLDS)
DEVICE = "cpu"
BATCH_SIZE = 64
exp_name = "try0__00_en_ioen_0_bhe_corrected_trtestsplit_tuple_reduxdrop_nobias_yeslog_standard4all_dbscan2_mid_30_randomsplit_FORCEprojMASSall"
dir_path = os.path.dirname(os.path.realpath(__file__))
out_cols = {
    "E": "Energy (MeV/amu)",
    "SP": "Stopping power (MeV cm2/mg)"
}


def run_NN(
    projectile: str,
    target: str,
    emin: int = 0.001,
    emax: int = 10,
    npoints: int = 150,
    outdir: str = "./",
    plot: bool = True,
):
    """Compute NN prediction for electronic stopping power

    Parameters
    ----------
    projectile : str
        Projectile symbol
    target : str
        Target symbol
    emin : int, optional
        Minimum grid-energy value (MeV/amu), by default 0.001
    emax : int, optional
        Maximum grid-energy value (MeV/amu), by default 10
    npoints : int, optional
        Number of grid-points, by default 500
    outdir : str, optional
        _description_, by default "./"
    plot : bool, optional
        _description_, by default True
    """

    # Generate grid
    df = generate_custom_table(
        projectile,
        target,
        emin,
        emax,
        npoints,
    )

    df["projectile_mass"] = df["projectile"].apply(get_mass)

    df["target_ionisation"] = df["target"].apply(get_ionisation_projectile)

    df["projectile_Z"] = df["projectile"].apply(get_Z_projectile)

    df["target_mass"] = df["target"].apply(get_mass)

    df["Z_max"] = df["target"].apply(get_max_Z)

    columns = [
        "target_mass",
        "projectile_mass",
        "Z_max",
        "projectile_Z",
        "normalized_energy",
        "target_ionisation",
    ]
    df[columns] = df[columns].astype(float)

    # Transform to logarithmic incident energy
    df_log = df.copy()
    df_log["normalized_energy"] = np.log(df["normalized_energy"].values)
    params = {"exp_name": exp_name, "model_dir": f"{dir_path}/data"}

    # Averaging on multiple SEEDS
    for seed in SEED:
        oof_ = run_k_fold(
            df_log,
            NFOLDS,
            seed,
            device=DEVICE,
            verbose=True,
            **params
        )

    for fold in range(NFOLDS):
        df_log[f"pred_{fold}"] = oof_[:, fold]

    df["stopping_power"] = np.mean(oof_, axis=1)
    df["system"] = df["projectile"] + "_" + df["target"]
    for tup in df["system"].unique():
        df_tup = df.loc[df["system"] == tup]
    df_tup = df_tup.dropna(axis=0, subset=['stopping_power'])
    if len(df_tup) == 0:
        raise ValueError(f"No values found for {projectile } + {target} collisional system")

    # Save dataframe with prediction to file
    filepath = os.path.join(outdir, f"{projectile + target}_prediction.dat")
    df_out = pd.DataFrame(
        {
            out_cols["E"]: df_tup["normalized_energy"],
            out_cols["SP"]: df_tup["stopping_power"]
        }
    )

    # 1. Remove negative stopping power interpolations
    try:
        df_out = df_out[df_out[out_cols["SP"]] >= 0]
    except:
        pass

    if len(df_out) != len(df_tup):
        new_emin = df_out.iloc[0][0]
        print(f"emin: {emin} => {new_emin:.4f}")

    df_out.to_csv(filepath, index=False, sep='\t')

    # Plot prediction
    if plot is True:
        plot_prediction(projectile, target, df_out)


def plot_prediction(projectile, target, df):
    e = out_cols['E']
    sp = out_cols['SP']
    title = ' '.join([projectile, "on", target])
    fig, ax = plt.subplots(1, 1, figsize=(8 * 1.1, 6 * 1.1))
    ax.scatter(df[e], df[sp], marker='.')
    ax.set_title(title, fontsize=20)
    ax.set_xscale("log")
    ax.set_xlabel(r"Energy (MeV/amu)", fontsize=18)
    ax.set_ylabel(r"Electronic Stopping Power (MeV cm$^2$/mg)", fontsize=18)
    ax.tick_params(axis='both', labelsize=14)
    plt.show()



import numpy as np
import torch
import joblib

from .dataset import TestDataset
from .SPNN import Model
from .utils import seed_everything


def inference_fn(model, dataloader, device):
    """
    Given a model a data loader and a target device executes inference with
    the model on the dataloader
    """
    model.eval()
    preds = []

    for data in dataloader:
        inputs = data["x"].to(device)

        with torch.no_grad():
            outputs = model(inputs)

        preds.append(outputs.detach().cpu().numpy())

    preds = np.concatenate(preds)

    return preds


def run_inference(
    X_valid,
    fold,
    seed,
    device,
    verbose=False,
    model_dir="/model",
    exp_name="default_exp",
):
    """Executes inference on X_valid dataset"""

    seed_everything(seed)

    scaler = joblib.load(f"{model_dir}/preprocess/std_scaler.zip")

    X_valid = scaler.transform(X_valid)

    valid_dataset = TestDataset(X_valid)

    validloader = torch.utils.data.DataLoader(
        valid_dataset, batch_size=256, shuffle=False
    )

    model = Model(num_features=X_valid.shape[1], num_targets=1)

    model.to(device)

    model.load_state_dict(
        torch.load(
            f"{model_dir}/weights/FOLD{fold}_{exp_name}.pth",
            map_location=torch.device(device),
        )
    )

    oof = inference_fn(model, validloader, device)

    return oof
